@@ -94,12 +94,13 @@ unsigned int casual(unsigned int numOr) {
 
 unsigned int calculate_expression(unsigned int start, unsigned int id_prod, unsigned char type) {
 	unsigned char code;
-	int result1, result2, result;
+	register unsigned char i=0, j=0;
+	int result1, result2, result[MAX_RECURSION];
 	
 	players[playerCount]->productions[id_prod]->ec=start;
 	while((code=players[playerCount]->productions[id_prod]->exp->expcode[players[playerCount]->productions[id_prod]->ec][0])!=_ENDEXP) {
 		switch(code) {
-			case _OR:
+			case _AND:
 				switch(players[playerCount]->productions[id_prod]->exp->expcode[players[playerCount]->productions[id_prod]->ec][1]) {
 					case 0:
 						result1=players[playerCount]->productions[id_prod]->exp->expcode[players[playerCount]->productions[id_prod]->ec][2];
@@ -108,7 +109,7 @@ unsigned int calculate_expression(unsigned int start, unsigned int id_prod, unsi
 						result1=players[playerCount]->local_vars[players[playerCount]->productions[id_prod]->exp->expcode[players[playerCount]->productions[id_prod]->ec][2]].name?players[playerCount]->local_vars[players[playerCount]->productions[id_prod]->exp->expcode[players[playerCount]->productions[id_prod]->ec][2]].value:global_vars[players[playerCount]->productions[id_prod]->exp->expcode[players[playerCount]->productions[id_prod]->ec][2]].value;
 						break;
 					case 2:
-						result1=result;
+						result1=result[j++];
 				}
 				switch(players[playerCount]->productions[id_prod]->exp->expcode[players[playerCount]->productions[id_prod]->ec][3]) {
 					case 0:
@@ -118,9 +119,32 @@ unsigned int calculate_expression(unsigned int start, unsigned int id_prod, unsi
 						result2=players[playerCount]->local_vars[players[playerCount]->productions[id_prod]->exp->expcode[players[playerCount]->productions[id_prod]->ec][4]].name?players[playerCount]->local_vars[players[playerCount]->productions[id_prod]->exp->expcode[players[playerCount]->productions[id_prod]->ec][4]].value:global_vars[players[playerCount]->productions[id_prod]->exp->expcode[players[playerCount]->productions[id_prod]->ec][4]].value;
 						break;
 					case 2:
-						result2=result;
+						result2=result[j++];
 				}
-				result=result1|result2;			
+				result[i++]=result1 && result2;
+				break;
+			case _OR:
+				switch(players[playerCount]->productions[id_prod]->exp->expcode[players[playerCount]->productions[id_prod]->ec][1]) {
+					case 0:
+						result1=players[playerCount]->productions[id_prod]->exp->expcode[players[playerCount]->productions[id_prod]->ec][2];
+						break;
+					case 1:
+						result1=players[playerCount]->local_vars[players[playerCount]->productions[id_prod]->exp->expcode[players[playerCount]->productions[id_prod]->ec][2]].name?players[playerCount]->local_vars[players[playerCount]->productions[id_prod]->exp->expcode[players[playerCount]->productions[id_prod]->ec][2]].value:global_vars[players[playerCount]->productions[id_prod]->exp->expcode[players[playerCount]->productions[id_prod]->ec][2]].value;
+						break;
+					case 2:
+						result1=result[j++];
+				}
+				switch(players[playerCount]->productions[id_prod]->exp->expcode[players[playerCount]->productions[id_prod]->ec][3]) {
+					case 0:
+						result2=players[playerCount]->productions[id_prod]->exp->expcode[players[playerCount]->productions[id_prod]->ec][4];
+						break;
+					case 1:
+						result2=players[playerCount]->local_vars[players[playerCount]->productions[id_prod]->exp->expcode[players[playerCount]->productions[id_prod]->ec][4]].name?players[playerCount]->local_vars[players[playerCount]->productions[id_prod]->exp->expcode[players[playerCount]->productions[id_prod]->ec][4]].value:global_vars[players[playerCount]->productions[id_prod]->exp->expcode[players[playerCount]->productions[id_prod]->ec][4]].value;
+						break;
+					case 2:
+						result2=result[j++];
+				}
+				result[i++]=result1 || result2;			
 				break;
 			case _NOT:
 				switch(players[playerCount]->productions[id_prod]->exp->expcode[players[playerCount]->productions[id_prod]->ec][1]) {
@@ -131,9 +155,9 @@ unsigned int calculate_expression(unsigned int start, unsigned int id_prod, unsi
 						result1=players[playerCount]->local_vars[players[playerCount]->productions[id_prod]->exp->expcode[players[playerCount]->productions[id_prod]->ec][2]].name?players[playerCount]->local_vars[players[playerCount]->productions[id_prod]->exp->expcode[players[playerCount]->productions[id_prod]->ec][2]].value:global_vars[players[playerCount]->productions[id_prod]->exp->expcode[players[playerCount]->productions[id_prod]->ec][2]].value;
 						break;
 					case 2:
-						result1=result;
+						result1=result[j++];
 				}			
-				result=!result1;
+				result[i++]=!result1;
 				break;
 			case _EQ:
 				switch(players[playerCount]->productions[id_prod]->exp->expcode[players[playerCount]->productions[id_prod]->ec][1]) {
@@ -144,7 +168,7 @@ unsigned int calculate_expression(unsigned int start, unsigned int id_prod, unsi
 						result1=players[playerCount]->local_vars[players[playerCount]->productions[id_prod]->exp->expcode[players[playerCount]->productions[id_prod]->ec][2]].name?players[playerCount]->local_vars[players[playerCount]->productions[id_prod]->exp->expcode[players[playerCount]->productions[id_prod]->ec][2]].value:global_vars[players[playerCount]->productions[id_prod]->exp->expcode[players[playerCount]->productions[id_prod]->ec][2]].value;
 						break;
 					case 2:
-						result1=result;
+						result1=result[j++];
 				}
 				switch(players[playerCount]->productions[id_prod]->exp->expcode[players[playerCount]->productions[id_prod]->ec][3]) {
 					case 0:
@@ -154,9 +178,9 @@ unsigned int calculate_expression(unsigned int start, unsigned int id_prod, unsi
 						result2=players[playerCount]->local_vars[players[playerCount]->productions[id_prod]->exp->expcode[players[playerCount]->productions[id_prod]->ec][4]].name?players[playerCount]->local_vars[players[playerCount]->productions[id_prod]->exp->expcode[players[playerCount]->productions[id_prod]->ec][4]].value:global_vars[players[playerCount]->productions[id_prod]->exp->expcode[players[playerCount]->productions[id_prod]->ec][4]].value;
 						break;
 					case 2:
-						result2=result;
+						result2=result[j++];
 				}
-				result=result1==result2;			
+				result[i++]=result1==result2;			
 				break;
 			case _NE:
 				switch(players[playerCount]->productions[id_prod]->exp->expcode[players[playerCount]->productions[id_prod]->ec][1]) {
@@ -167,7 +191,7 @@ unsigned int calculate_expression(unsigned int start, unsigned int id_prod, unsi
 						result1=players[playerCount]->local_vars[players[playerCount]->productions[id_prod]->exp->expcode[players[playerCount]->productions[id_prod]->ec][2]].name?players[playerCount]->local_vars[players[playerCount]->productions[id_prod]->exp->expcode[players[playerCount]->productions[id_prod]->ec][2]].value:global_vars[players[playerCount]->productions[id_prod]->exp->expcode[players[playerCount]->productions[id_prod]->ec][2]].value;
 						break;
 					case 2:
-						result1=result;
+						result1=result[j++];
 				}
 				switch(players[playerCount]->productions[id_prod]->exp->expcode[players[playerCount]->productions[id_prod]->ec][3]) {
 					case 0:
@@ -177,9 +201,9 @@ unsigned int calculate_expression(unsigned int start, unsigned int id_prod, unsi
 						result2=players[playerCount]->local_vars[players[playerCount]->productions[id_prod]->exp->expcode[players[playerCount]->productions[id_prod]->ec][4]].name?players[playerCount]->local_vars[players[playerCount]->productions[id_prod]->exp->expcode[players[playerCount]->productions[id_prod]->ec][4]].value:global_vars[players[playerCount]->productions[id_prod]->exp->expcode[players[playerCount]->productions[id_prod]->ec][4]].value;
 						break;
 					case 2:
-						result2=result;
+						result2=result[j++];
 				}
-				result=result1!=result2;			
+				result[i++]=result1!=result2;			
 				break;
 			case _LT:
 				switch(players[playerCount]->productions[id_prod]->exp->expcode[players[playerCount]->productions[id_prod]->ec][1]) {
@@ -190,7 +214,7 @@ unsigned int calculate_expression(unsigned int start, unsigned int id_prod, unsi
 						result1=players[playerCount]->local_vars[players[playerCount]->productions[id_prod]->exp->expcode[players[playerCount]->productions[id_prod]->ec][2]].name?players[playerCount]->local_vars[players[playerCount]->productions[id_prod]->exp->expcode[players[playerCount]->productions[id_prod]->ec][2]].value:global_vars[players[playerCount]->productions[id_prod]->exp->expcode[players[playerCount]->productions[id_prod]->ec][2]].value;
 						break;
 					case 2:
-						result1=result;
+						result1=result[j++];
 				}
 				switch(players[playerCount]->productions[id_prod]->exp->expcode[players[playerCount]->productions[id_prod]->ec][3]) {
 					case 0:
@@ -200,9 +224,9 @@ unsigned int calculate_expression(unsigned int start, unsigned int id_prod, unsi
 						result2=players[playerCount]->local_vars[players[playerCount]->productions[id_prod]->exp->expcode[players[playerCount]->productions[id_prod]->ec][4]].name?players[playerCount]->local_vars[players[playerCount]->productions[id_prod]->exp->expcode[players[playerCount]->productions[id_prod]->ec][4]].value:global_vars[players[playerCount]->productions[id_prod]->exp->expcode[players[playerCount]->productions[id_prod]->ec][4]].value;
 						break;
 					case 2:
-						result2=result;
+						result2=result[j++];
 				}
-				result=result1<result2;			
+				result[i++]=result1<result2;			
 				break;
 			case _LE:
 				switch(players[playerCount]->productions[id_prod]->exp->expcode[players[playerCount]->productions[id_prod]->ec][1]) {
@@ -213,7 +237,7 @@ unsigned int calculate_expression(unsigned int start, unsigned int id_prod, unsi
 						result1=players[playerCount]->local_vars[players[playerCount]->productions[id_prod]->exp->expcode[players[playerCount]->productions[id_prod]->ec][2]].name?players[playerCount]->local_vars[players[playerCount]->productions[id_prod]->exp->expcode[players[playerCount]->productions[id_prod]->ec][2]].value:global_vars[players[playerCount]->productions[id_prod]->exp->expcode[players[playerCount]->productions[id_prod]->ec][2]].value;
 						break;
 					case 2:
-						result1=result;
+						result1=result[j++];
 				}
 				switch(players[playerCount]->productions[id_prod]->exp->expcode[players[playerCount]->productions[id_prod]->ec][3]) {
 					case 0:
@@ -223,9 +247,9 @@ unsigned int calculate_expression(unsigned int start, unsigned int id_prod, unsi
 						result2=players[playerCount]->local_vars[players[playerCount]->productions[id_prod]->exp->expcode[players[playerCount]->productions[id_prod]->ec][4]].name?players[playerCount]->local_vars[players[playerCount]->productions[id_prod]->exp->expcode[players[playerCount]->productions[id_prod]->ec][4]].value:global_vars[players[playerCount]->productions[id_prod]->exp->expcode[players[playerCount]->productions[id_prod]->ec][4]].value;
 						break;
 					case 2:
-						result2=result;
+						result2=result[j++];
 				}
-				result=result1<=result2;			
+				result[i++]=result1<=result2;			
 				break;
 			case _GT:
 				switch(players[playerCount]->productions[id_prod]->exp->expcode[players[playerCount]->productions[id_prod]->ec][1]) {
@@ -236,7 +260,7 @@ unsigned int calculate_expression(unsigned int start, unsigned int id_prod, unsi
 						result1=players[playerCount]->local_vars[players[playerCount]->productions[id_prod]->exp->expcode[players[playerCount]->productions[id_prod]->ec][2]].name?players[playerCount]->local_vars[players[playerCount]->productions[id_prod]->exp->expcode[players[playerCount]->productions[id_prod]->ec][2]].value:global_vars[players[playerCount]->productions[id_prod]->exp->expcode[players[playerCount]->productions[id_prod]->ec][2]].value;
 						break;
 					case 2:
-						result1=result;
+						result1=result[j++];
 				}
 				switch(players[playerCount]->productions[id_prod]->exp->expcode[players[playerCount]->productions[id_prod]->ec][3]) {
 					case 0:
@@ -246,9 +270,9 @@ unsigned int calculate_expression(unsigned int start, unsigned int id_prod, unsi
 						result2=players[playerCount]->local_vars[players[playerCount]->productions[id_prod]->exp->expcode[players[playerCount]->productions[id_prod]->ec][4]].name?players[playerCount]->local_vars[players[playerCount]->productions[id_prod]->exp->expcode[players[playerCount]->productions[id_prod]->ec][4]].value:global_vars[players[playerCount]->productions[id_prod]->exp->expcode[players[playerCount]->productions[id_prod]->ec][4]].value;
 						break;
 					case 2:
-						result2=result;
+						result2=result[j++];
 				}
-				result=result1>result2;			
+				result[i++]=result1>result2;			
 				break;
 			case _GE:
 				switch(players[playerCount]->productions[id_prod]->exp->expcode[players[playerCount]->productions[id_prod]->ec][1]) {
@@ -259,7 +283,7 @@ unsigned int calculate_expression(unsigned int start, unsigned int id_prod, unsi
 						result1=players[playerCount]->local_vars[players[playerCount]->productions[id_prod]->exp->expcode[players[playerCount]->productions[id_prod]->ec][2]].name?players[playerCount]->local_vars[players[playerCount]->productions[id_prod]->exp->expcode[players[playerCount]->productions[id_prod]->ec][2]].value:global_vars[players[playerCount]->productions[id_prod]->exp->expcode[players[playerCount]->productions[id_prod]->ec][2]].value;
 						break;
 					case 2:
-						result1=result;
+						result1=result[j++];
 				}
 				switch(players[playerCount]->productions[id_prod]->exp->expcode[players[playerCount]->productions[id_prod]->ec][3]) {
 					case 0:
@@ -269,9 +293,9 @@ unsigned int calculate_expression(unsigned int start, unsigned int id_prod, unsi
 						result2=players[playerCount]->local_vars[players[playerCount]->productions[id_prod]->exp->expcode[players[playerCount]->productions[id_prod]->ec][4]].name?players[playerCount]->local_vars[players[playerCount]->productions[id_prod]->exp->expcode[players[playerCount]->productions[id_prod]->ec][4]].value:global_vars[players[playerCount]->productions[id_prod]->exp->expcode[players[playerCount]->productions[id_prod]->ec][4]].value;
 						break;
 					case 2:
-						result2=result;
+						result2=result[j++];
 				}
-				result=result1>=result2;			
+				result[i++]=result1>=result2;			
 				break;
 			case _MOV:
 				switch(players[playerCount]->productions[id_prod]->exp->expcode[players[playerCount]->productions[id_prod]->ec][2]) {
@@ -287,10 +311,10 @@ unsigned int calculate_expression(unsigned int start, unsigned int id_prod, unsi
 						break;
 					case 2:
 						players[playerCount]->local_vars[players[playerCount]->productions[id_prod]->exp->expcode[players[playerCount]->productions[id_prod]->ec][1]].name?
-						(result1=players[playerCount]->local_vars[players[playerCount]->productions[id_prod]->exp->expcode[players[playerCount]->productions[id_prod]->ec][1]].value=result):
-						(result1=global_vars[players[playerCount]->productions[id_prod]->exp->expcode[players[playerCount]->productions[id_prod]->ec][1]].value=result);
+						(result1=players[playerCount]->local_vars[players[playerCount]->productions[id_prod]->exp->expcode[players[playerCount]->productions[id_prod]->ec][1]].value=result[j++]):
+						(result1=global_vars[players[playerCount]->productions[id_prod]->exp->expcode[players[playerCount]->productions[id_prod]->ec][1]].value=result[j++]);
 				}
-				result=result1;		
+				result[i++]=result1;		
 				break;
 			case _ADD:
 				switch(players[playerCount]->productions[id_prod]->exp->expcode[players[playerCount]->productions[id_prod]->ec][1]) {
@@ -301,7 +325,7 @@ unsigned int calculate_expression(unsigned int start, unsigned int id_prod, unsi
 						result1=players[playerCount]->local_vars[players[playerCount]->productions[id_prod]->exp->expcode[players[playerCount]->productions[id_prod]->ec][2]].name?players[playerCount]->local_vars[players[playerCount]->productions[id_prod]->exp->expcode[players[playerCount]->productions[id_prod]->ec][2]].value:global_vars[players[playerCount]->productions[id_prod]->exp->expcode[players[playerCount]->productions[id_prod]->ec][2]].value;
 						break;
 					case 2:
-						result1=result;
+						result1=result[j++];
 				}
 				switch(players[playerCount]->productions[id_prod]->exp->expcode[players[playerCount]->productions[id_prod]->ec][3]) {
 					case 0:
@@ -311,9 +335,9 @@ unsigned int calculate_expression(unsigned int start, unsigned int id_prod, unsi
 						result2=players[playerCount]->local_vars[players[playerCount]->productions[id_prod]->exp->expcode[players[playerCount]->productions[id_prod]->ec][4]].name?players[playerCount]->local_vars[players[playerCount]->productions[id_prod]->exp->expcode[players[playerCount]->productions[id_prod]->ec][4]].value:global_vars[players[playerCount]->productions[id_prod]->exp->expcode[players[playerCount]->productions[id_prod]->ec][4]].value;
 						break;
 					case 2:
-						result2=result;
+						result2=result[j++];
 				}
-				result=result1+result2;
+				result[i++]=result1+result2;
 				break;
 			case _SUB:
 				switch(players[playerCount]->productions[id_prod]->exp->expcode[players[playerCount]->productions[id_prod]->ec][1]) {
@@ -324,7 +348,7 @@ unsigned int calculate_expression(unsigned int start, unsigned int id_prod, unsi
 						result1=players[playerCount]->local_vars[players[playerCount]->productions[id_prod]->exp->expcode[players[playerCount]->productions[id_prod]->ec][2]].name?players[playerCount]->local_vars[players[playerCount]->productions[id_prod]->exp->expcode[players[playerCount]->productions[id_prod]->ec][2]].value:global_vars[players[playerCount]->productions[id_prod]->exp->expcode[players[playerCount]->productions[id_prod]->ec][2]].value;
 						break;
 					case 2:
-						result1=result;
+						result1=result[j++];
 				}
 				switch(players[playerCount]->productions[id_prod]->exp->expcode[players[playerCount]->productions[id_prod]->ec][3]) {
 					case 0:
@@ -334,9 +358,9 @@ unsigned int calculate_expression(unsigned int start, unsigned int id_prod, unsi
 						result2=players[playerCount]->local_vars[players[playerCount]->productions[id_prod]->exp->expcode[players[playerCount]->productions[id_prod]->ec][4]].name?players[playerCount]->local_vars[players[playerCount]->productions[id_prod]->exp->expcode[players[playerCount]->productions[id_prod]->ec][4]].value:global_vars[players[playerCount]->productions[id_prod]->exp->expcode[players[playerCount]->productions[id_prod]->ec][4]].value;
 						break;
 					case 2:
-						result2=result;
+						result2=result[j++];
 				}			
-				result=result1-result2;
+				result[i++]=result1-result2;
 				break;
 			case _MUL:
 				switch(players[playerCount]->productions[id_prod]->exp->expcode[players[playerCount]->productions[id_prod]->ec][1]) {
@@ -347,7 +371,7 @@ unsigned int calculate_expression(unsigned int start, unsigned int id_prod, unsi
 						result1=players[playerCount]->local_vars[players[playerCount]->productions[id_prod]->exp->expcode[players[playerCount]->productions[id_prod]->ec][2]].name?players[playerCount]->local_vars[players[playerCount]->productions[id_prod]->exp->expcode[players[playerCount]->productions[id_prod]->ec][2]].value:global_vars[players[playerCount]->productions[id_prod]->exp->expcode[players[playerCount]->productions[id_prod]->ec][2]].value;
 						break;
 					case 2:
-						result1=result;
+						result1=result[j++];
 				}
 				switch(players[playerCount]->productions[id_prod]->exp->expcode[players[playerCount]->productions[id_prod]->ec][3]) {
 					case 0:
@@ -357,9 +381,9 @@ unsigned int calculate_expression(unsigned int start, unsigned int id_prod, unsi
 						result2=players[playerCount]->local_vars[players[playerCount]->productions[id_prod]->exp->expcode[players[playerCount]->productions[id_prod]->ec][4]].name?players[playerCount]->local_vars[players[playerCount]->productions[id_prod]->exp->expcode[players[playerCount]->productions[id_prod]->ec][4]].value:global_vars[players[playerCount]->productions[id_prod]->exp->expcode[players[playerCount]->productions[id_prod]->ec][4]].value;
 						break;
 					case 2:
-						result2=result;
+						result2=result[j++];
 				}			
-				result=result1*result2;			
+				result[i++]=result1*result2;			
 				break;
 			case _DIV:
 				switch(players[playerCount]->productions[id_prod]->exp->expcode[players[playerCount]->productions[id_prod]->ec][1]) {
@@ -370,7 +394,7 @@ unsigned int calculate_expression(unsigned int start, unsigned int id_prod, unsi
 						result1=players[playerCount]->local_vars[players[playerCount]->productions[id_prod]->exp->expcode[players[playerCount]->productions[id_prod]->ec][2]].name?players[playerCount]->local_vars[players[playerCount]->productions[id_prod]->exp->expcode[players[playerCount]->productions[id_prod]->ec][2]].value:global_vars[players[playerCount]->productions[id_prod]->exp->expcode[players[playerCount]->productions[id_prod]->ec][2]].value;
 						break;
 					case 2:
-						result1=result;
+						result1=result[j++];
 				}
 				switch(players[playerCount]->productions[id_prod]->exp->expcode[players[playerCount]->productions[id_prod]->ec][3]) {
 					case 0:
@@ -380,9 +404,9 @@ unsigned int calculate_expression(unsigned int start, unsigned int id_prod, unsi
 						result2=players[playerCount]->local_vars[players[playerCount]->productions[id_prod]->exp->expcode[players[playerCount]->productions[id_prod]->ec][4]].name?players[playerCount]->local_vars[players[playerCount]->productions[id_prod]->exp->expcode[players[playerCount]->productions[id_prod]->ec][4]].value:global_vars[players[playerCount]->productions[id_prod]->exp->expcode[players[playerCount]->productions[id_prod]->ec][4]].value;
 						break;
 					case 2:
-						result2=result;
+						result2=result[j++];
 				}			
-				result=result1/result2;
+				result[i++]=result1/result2;
 				break;
 			case _MOD:
 				switch(players[playerCount]->productions[id_prod]->exp->expcode[players[playerCount]->productions[id_prod]->ec][1]) {
@@ -393,7 +417,7 @@ unsigned int calculate_expression(unsigned int start, unsigned int id_prod, unsi
 						result1=players[playerCount]->local_vars[players[playerCount]->productions[id_prod]->exp->expcode[players[playerCount]->productions[id_prod]->ec][2]].name?players[playerCount]->local_vars[players[playerCount]->productions[id_prod]->exp->expcode[players[playerCount]->productions[id_prod]->ec][2]].value:global_vars[players[playerCount]->productions[id_prod]->exp->expcode[players[playerCount]->productions[id_prod]->ec][2]].value;
 						break;
 					case 2:
-						result1=result;
+						result1=result[j++];
 				}
 				switch(players[playerCount]->productions[id_prod]->exp->expcode[players[playerCount]->productions[id_prod]->ec][3]) {
 					case 0:
@@ -403,19 +427,19 @@ unsigned int calculate_expression(unsigned int start, unsigned int id_prod, unsi
 						result2=players[playerCount]->local_vars[players[playerCount]->productions[id_prod]->exp->expcode[players[playerCount]->productions[id_prod]->ec][4]].name?players[playerCount]->local_vars[players[playerCount]->productions[id_prod]->exp->expcode[players[playerCount]->productions[id_prod]->ec][4]].value:global_vars[players[playerCount]->productions[id_prod]->exp->expcode[players[playerCount]->productions[id_prod]->ec][4]].value;
 						break;
 					case 2:
-						result2=result;
+						result2=result[j++];
 				}			
-				result=result1%result2;
+				result[i++]=result1%result2;
 				break;				
 			case _INC:
 				result1=players[playerCount]->local_vars[players[playerCount]->productions[id_prod]->exp->expcode[players[playerCount]->productions[id_prod]->ec][1]].name?players[playerCount]->local_vars[players[playerCount]->productions[id_prod]->exp->expcode[players[playerCount]->productions[id_prod]->ec][1]].value:global_vars[players[playerCount]->productions[id_prod]->exp->expcode[players[playerCount]->productions[id_prod]->ec][1]].value;
 				result2=players[playerCount]->productions[id_prod]->exp->expcode[players[playerCount]->productions[id_prod]->ec][2];
-				result=result1+result2;
+				result[i++]=result1+result2;
 				break;
 			case _DEC:
 				result1=players[playerCount]->local_vars[players[playerCount]->productions[id_prod]->exp->expcode[players[playerCount]->productions[id_prod]->ec][1]].name?players[playerCount]->local_vars[players[playerCount]->productions[id_prod]->exp->expcode[players[playerCount]->productions[id_prod]->ec][1]].value:global_vars[players[playerCount]->productions[id_prod]->exp->expcode[players[playerCount]->productions[id_prod]->ec][1]].value;
 				result2=players[playerCount]->productions[id_prod]->exp->expcode[players[playerCount]->productions[id_prod]->ec][2];
-				result=result1-result2;						
+				result[i++]=result1-result2;						
 				break;
 			case _RND:
 				switch(players[playerCount]->productions[id_prod]->exp->expcode[players[playerCount]->productions[id_prod]->ec][1]) {
@@ -426,9 +450,9 @@ unsigned int calculate_expression(unsigned int start, unsigned int id_prod, unsi
 						result1=players[playerCount]->local_vars[players[playerCount]->productions[id_prod]->exp->expcode[players[playerCount]->productions[id_prod]->ec][2]].name?players[playerCount]->local_vars[players[playerCount]->productions[id_prod]->exp->expcode[players[playerCount]->productions[id_prod]->ec][2]].value:global_vars[players[playerCount]->productions[id_prod]->exp->expcode[players[playerCount]->productions[id_prod]->ec][2]].value;
 						break;
 					case 2:
-						result1=result;
+						result1=result[j++];
 				}
-				result=rand()%result1;
+				result[i++]=rand()%result1;
 				break;
 			case _UMIN:
 				switch(players[playerCount]->productions[id_prod]->exp->expcode[players[playerCount]->productions[id_prod]->ec][1]) {
@@ -439,33 +463,33 @@ unsigned int calculate_expression(unsigned int start, unsigned int id_prod, unsi
 						result1=players[playerCount]->local_vars[players[playerCount]->productions[id_prod]->exp->expcode[players[playerCount]->productions[id_prod]->ec][2]].name?players[playerCount]->local_vars[players[playerCount]->productions[id_prod]->exp->expcode[players[playerCount]->productions[id_prod]->ec][2]].value:global_vars[players[playerCount]->productions[id_prod]->exp->expcode[players[playerCount]->productions[id_prod]->ec][2]].value;
 						break;
 					case 2:
-						result1=result;
+						result1=result[j++];
 				}			
-				result=-result1;
+				result[i++]=-result1;
 		}
 	if(players[playerCount]->productions[id_prod]->ec+1<DIM_EXP)	
 		players[playerCount]->productions[id_prod]->ec++;	
 	}
 	switch(type) {
 		case OCT:
-			if(result<-2)
-				result=-2;
-			if(result>8)
-				result=8;
+			if(result[i-1]<-2)
+				result[i-1]=-2;
+			if(result[i-1]>8)
+				result[i-1]=8;
 			break;
 		case VEL:
 		case REL:
 		case BYTEDATA:
-			if(result<-127)
-				result=-127;
-			if(result>127)
-				result=127;		
+			if(result[i-1]<-127)
+				result[i-1]=-127;
+			if(result[i-1]>127)
+				result[i-1]=127;		
 			break;
 		case DUR:
-			if(result<0)
-				result=0;	
+			if(result[i-1]<0)
+				result[i-1]=0;	
 	}
-	return result;
+	return result[i-1];
 }
 
 void process_rest(unsigned int id_prod, unsigned char isChord) {
@@ -610,19 +634,24 @@ void process_note(unsigned int id_prod, unsigned char isChord) {
 	}			
 }
 
-unsigned char chomsky_generator(unsigned int nt, unsigned int iteration, unsigned char isChord) {
-	unsigned char code, stop=0, rep[MAX_RECURSION][2];
-	unsigned int it, caso, reg, *pointer, tmp;
+unsigned char chomsky_generator(unsigned int nt, unsigned char isChord) {
+	unsigned char code, rep[MAX_RECURSION][2];
+	unsigned int it, caso, casoAlt, reg, *pointer, tmp;
 	register unsigned int i=0, j=0, k, w;
 	register char note_chord_counter=0;
 	int sum_trn=0, sum_inv=0, y;
 	
-	if(!iteration)
+	//(!players[playerCount]->productions[nt]->visited)?(players[playerCount]->productions[nt]->visited++):(players[playerCount]->productions[nt]->iterationCounter--);
+	if(!players[playerCount]->productions[nt]->visited)
+		players[playerCount]->productions[nt]->visited=++level;
+	if(!players[playerCount]->productions[nt]->iterationCounter) {
+		players[playerCount]->iterations?(players[playerCount]->productions[nt]->iterationCounter=players[playerCount]->iterations):(players[playerCount]->productions[nt]->iterationCounter=global_iterations);	
 		return 0;
+	}
 	players[playerCount]->productions[nt]->numOr>1?(caso=casual(players[playerCount]->productions[nt]->numOr)):(caso=1);
+	players[playerCount]->productions[nt]->numOrAlt>1?(casoAlt=casual(players[playerCount]->productions[nt]->numOrAlt)):(casoAlt=1);
 	if(debugOption && players[playerCount]->productions[nt]->numOr>1)
-		printf("this is a non-deterministic production with %d alternatives. I choose: %d\n", players[playerCount]->productions[nt]->numOr, caso);	//debug
-	players[playerCount]->iterations?(it=players[playerCount]->iterations):(it=global_iterations);	
+		printf("this is a non-deterministic production with %d alternatives. I choose: %d\n", players[playerCount]->productions[nt]->numOr, caso);	//debug	
 	players[playerCount]->productions[nt]->cc=0;
 	//printf("# of %s production: %d\n", players[playerCount]->productions[nt]->name, players[playerCount]->productions[nt]->numOr);	//debug
 	while(code=players[playerCount]->productions[nt]->midicode[players[playerCount]->productions[nt]->cc][0]) {
@@ -642,18 +671,36 @@ unsigned char chomsky_generator(unsigned int nt, unsigned int iteration, unsigne
 					players[playerCount]->productions[nt]->cc--;
 				}
 				break;
+			case _PRD_ALT:
+				if(players[playerCount]->productions[nt]->midicode[players[playerCount]->productions[nt]->cc][1]!=casoAlt) {
+					if(players[playerCount]->productions[nt]->cc+1<DIM_MIDI_CODE)
+						players[playerCount]->productions[nt]->cc++;
+					else
+						break;
+					while(players[playerCount]->productions[nt]->midicode[players[playerCount]->productions[nt]->cc][0]!=_PRD_ALT) {
+						if(players[playerCount]->productions[nt]->cc+1<DIM_MIDI_CODE)
+							players[playerCount]->productions[nt]->cc++;
+						else
+							break;
+					}
+					players[playerCount]->productions[nt]->cc--;
+				}			
+				break;
 			case _CHK:
 				if(!calculate_expression(players[playerCount]->productions[nt]->midicode[players[playerCount]->productions[nt]->cc][1], nt, 0))
-					return 1;
+					if(!players[playerCount]->productions[nt]->midicode[players[playerCount]->productions[nt]->cc][2])
+						return 1;	
+					else
+						players[playerCount]->productions[nt]->cc=players[playerCount]->productions[nt]->midicode[players[playerCount]->productions[nt]->cc][2]-1;
 				break;
+			case _ELSE:
+				return 0;
 			case _GOTO:
 				reg=players[playerCount]->productions[nt]->cc;
-				nt==players[playerCount]->productions[nt]->midicode[players[playerCount]->productions[nt]->cc][1]?stop=chomsky_generator(players[playerCount]->productions[nt]->midicode[players[playerCount]->productions[nt]->cc][1], iteration-1, isChord):chomsky_generator(players[playerCount]->productions[nt]->midicode[players[playerCount]->productions[nt]->cc][1], it+1, isChord);
-				if(stop) {
-					iteration--;
-					players[playerCount]->productions[nt]->cc=reg;
-					continue;
-				}	
+				if(players[playerCount]->productions[players[playerCount]->productions[nt]->midicode[players[playerCount]->productions[nt]->cc][1]]->visited && players[playerCount]->productions[nt]->visited>=players[playerCount]->productions[players[playerCount]->productions[nt]->midicode[players[playerCount]->productions[nt]->cc][1]]->visited)
+					players[playerCount]->productions[players[playerCount]->productions[nt]->midicode[players[playerCount]->productions[nt]->cc][1]]->iterationCounter--;
+				chomsky_generator(players[playerCount]->productions[nt]->midicode[players[playerCount]->productions[nt]->cc][1], isChord);
+				players[playerCount]->productions[nt]->cc=reg;
 				break;
 			case _NOTEON:
 				process_note(nt, isChord);
@@ -961,19 +1008,41 @@ unsigned char chomsky_generator(unsigned int nt, unsigned int iteration, unsigne
 					note_chord_counter=0;
 				}
 				else
-					(*rtgd_buffer[rtgd_counter-1])[rtgd_note_count++][0]=_ENDCHORD;								
+					(*rtgd_buffer[rtgd_counter-1])[rtgd_note_count++][0]=_ENDCHORD;
+				break;
+			case _PRNT:
+				switch(players[playerCount]->productions[nt]->midicode[players[playerCount]->productions[nt]->cc][1]) {
+					case 0:
+						printf("%d", players[playerCount]->productions[nt]->midicode[players[playerCount]->productions[nt]->cc][2]);
+						break;
+					case 1:
+						printf("%d", players[playerCount]->local_vars[players[playerCount]->productions[nt]->midicode[players[playerCount]->productions[nt]->cc][2]].name?players[playerCount]->local_vars[players[playerCount]->productions[nt]->midicode[players[playerCount]->productions[nt]->cc][2]].value:global_vars[players[playerCount]->productions[nt]->midicode[players[playerCount]->productions[nt]->cc][2]].value);
+						break;
+					case _EXP:
+						printf("%d", calculate_expression(players[playerCount]->productions[nt]->midicode[players[playerCount]->productions[nt]->cc][2], nt, players[playerCount]->productions[nt]->midicode[players[playerCount]->productions[nt]->cc][3]));
+						break;
+					case _STR:
+						printf("%s", strings[players[playerCount]->productions[nt]->midicode[players[playerCount]->productions[nt]->cc][2]]);
+				}
 		}
 		if(players[playerCount]->productions[nt]->cc+1<DIM_MIDI_CODE)
 			players[playerCount]->productions[nt]->cc++;
 		else
 			break;
 	}
+	if(players[playerCount]->iterations) {
+		if(players[playerCount]->productions[nt]->iterationCounter<players[playerCount]->iterations)
+			players[playerCount]->productions[nt]->iterationCounter=players[playerCount]->iterations;
+	}
+	else
+		if(players[playerCount]->productions[nt]->iterationCounter<global_iterations)
+			players[playerCount]->productions[nt]->iterationCounter=global_iterations;
 	return 0;
 }
 
 unsigned char lindenmayer_prod_generator(unsigned int nt, unsigned int *buffer) {
 	unsigned char code, rep[MAX_RECURSION][2], isChord=0;
-	unsigned int caso, *pointer, tmp;
+	unsigned int caso, casoAlt, *pointer, tmp;
 	char string_buf[MAX_STR], str_temp_buf[MAX_STR];
 	register unsigned int i=0, j=0, k, w, counter=0;
 	register char note_chord_counter=0;
@@ -981,6 +1050,7 @@ unsigned char lindenmayer_prod_generator(unsigned int nt, unsigned int *buffer)
 	
 	memset(melop, 0, 2*MAX_RECURSION);
 	players[playerCount]->productions[nt]->numOr>1?(caso=casual(players[playerCount]->productions[nt]->numOr)):(caso=1);
+	players[playerCount]->productions[nt]->numOrAlt>1?(caso=casual(players[playerCount]->productions[nt]->numOrAlt)):(casoAlt=1);
 	if(debugOption && players[playerCount]->productions[nt]->numOr>1)
 		printf("this is a non-deterministic production with %d alternatives. I choose: %d\n", players[playerCount]->productions[nt]->numOr, caso);	//debug	
 	players[playerCount]->productions[nt]->cc=0;
@@ -1001,10 +1071,30 @@ unsigned char lindenmayer_prod_generator(unsigned int nt, unsigned int *buffer)
 					players[playerCount]->productions[nt]->cc--;
 				}				
 				break;
+			case _PRD_ALT:
+				if(players[playerCount]->productions[nt]->midicode[players[playerCount]->productions[nt]->cc][1]!=casoAlt) {
+					if(players[playerCount]->productions[nt]->cc+1<DIM_MIDI_CODE)
+						players[playerCount]->productions[nt]->cc++;
+					else
+						break;
+					while(players[playerCount]->productions[nt]->midicode[players[playerCount]->productions[nt]->cc][0]!=_PRD_ALT) {
+						if(players[playerCount]->productions[nt]->cc+1<DIM_MIDI_CODE)
+							players[playerCount]->productions[nt]->cc++;
+						else
+							break;
+					}
+					players[playerCount]->productions[nt]->cc--;
+				}			
+				break;				
 			case _CHK:
 				if(!calculate_expression(players[playerCount]->productions[nt]->midicode[players[playerCount]->productions[nt]->cc][1], nt, 0))
-					return 0;
+					if(!players[playerCount]->productions[nt]->midicode[players[playerCount]->productions[nt]->cc][2])
+						return 0;
+					else
+						players[playerCount]->productions[nt]->cc=players[playerCount]->productions[nt]->midicode[players[playerCount]->productions[nt]->cc][2]-1;
 				break;
+			case _ELSE:
+				return 0;				
 			case _NOTEON:
 				process_note(nt, isChord);
 				if(!rtgd_counter) {
@@ -1338,7 +1428,22 @@ unsigned char lindenmayer_prod_generator(unsigned int nt, unsigned int *buffer)
 					note_chord_counter=0;
 				}
 				else
-					(*rtgd_buffer[rtgd_counter-1])[rtgd_note_count++][0]=_ENDCHORD;					
+					(*rtgd_buffer[rtgd_counter-1])[rtgd_note_count++][0]=_ENDCHORD;
+				break;
+			case _PRNT:
+				switch(players[playerCount]->productions[nt]->midicode[players[playerCount]->productions[nt]->cc][1]) {
+					case 0:
+						printf("%d", players[playerCount]->productions[nt]->midicode[players[playerCount]->productions[nt]->cc][2]);
+						break;
+					case 1:
+						printf("%d", players[playerCount]->local_vars[players[playerCount]->productions[nt]->midicode[players[playerCount]->productions[nt]->cc][2]].name?players[playerCount]->local_vars[players[playerCount]->productions[nt]->midicode[players[playerCount]->productions[nt]->cc][2]].value:global_vars[players[playerCount]->productions[nt]->midicode[players[playerCount]->productions[nt]->cc][2]].value);
+						break;
+					case _EXP:
+						printf("%d", calculate_expression(players[playerCount]->productions[nt]->midicode[players[playerCount]->productions[nt]->cc][2], nt, players[playerCount]->productions[nt]->midicode[players[playerCount]->productions[nt]->cc][3]));
+						break;
+					case _STR:
+						printf("%s", strings[players[playerCount]->productions[nt]->midicode[players[playerCount]->productions[nt]->cc][2]]);
+				}		
 		}
 		if(players[playerCount]->productions[nt]->cc+1<DIM_MIDI_CODE)
 			players[playerCount]->productions[nt]->cc++;
@@ -1403,7 +1508,7 @@ int write_player_track(track) {
 		case 1:
 			if(debugOption)
 				print_local_params(playerCount);		//debug
-			chomsky_generator(hash("@composition", 0), iteration+1, 0);
+			chomsky_generator(hash("@composition", 0), 0);
 			break;
 		case 2:
 			if(debugOption)
@@ -1415,7 +1520,7 @@ int write_player_track(track) {
 				print_local_params(playerCount);		//debug
 			switch(grammarOption) {
 				case 1:
-					chomsky_generator(hash("@composition", 0), iteration+1, 0);
+					chomsky_generator(hash("@composition", 0), 0);
 					break;
 				case 2:
 					lindenmayer_generator(hash("axiom", 0), iteration);
@@ -1441,10 +1546,10 @@ int grammyvm() {
 		if(players[playerCount]) {
 			prev_duration=0;
 			if(!debugOption)
-				printf("Player %s is generating...", players[playerCount]->identifier);
+				printf("Player %s is generating...\n", players[playerCount]->identifier);
 			mfwrite(0, 1, global_resolution, midi);
 			if(!debugOption)
-				printf("ok\n");
+				printf("\n...ok\n");
 		}
 	}
 	if(!debugOption)
